/*
 * Oleg Orlov, 2013 (c) RLE encoding/decoding tool
 * C#, .net 2.0 by default, maybe upgraded to any version of .net framework
 * I have downgraded the .net version only for the compatibility aims
 * and for easy reproduction the program to any languages
 */

using System;

class RLE : IDisposable
{
    private string str_base, str_rle;

    private bool HasChar(ref string input)
    {
        bool status = false;

        for (int i = 0; i < input.Length; i++)
        {
            if (Char.IsLetter(input[i]))
            {
                status = true;
                break;
            }
        }

        return status;
    }

    internal string Encode(ref string input)
    {
        str_rle = null;
        str_base = input;

        for (int i = 0; i < str_base.Length; i++)
        {
            char symbol = str_base[i];
            int count = 1;

            for (int j = i; j < str_base.Length - 1; j++)
            {
                if (str_base[j + 1] != symbol) break;

                count++;
                i++;
            }

            if (count == 1) str_rle += symbol;
            else str_rle += count.ToString() + symbol;
        }

        return str_rle;
    }

    internal string Decode(ref string input)
    {
        str_rle = null;
        str_base = input;
        int radix = 0;

        for (int i = 0; i < str_base.Length; i++)
        {
            if (Char.IsNumber(str_base[i]))
            {
                radix++;
            }
            else
            {
                if (radix > 0)
                {
                    int value_repeat = Convert.ToInt32(str_base.Substring(i - radix, radix));

                    for (int j = 0; j < value_repeat; j++)
                    {
                        str_rle += str_base[i];
                    }

                    radix = 0;
                }
                else if (radix == 0)
                {
                    str_rle += str_base[i];
                }
            }
        }

        if (str_rle == null || !HasChar(ref str_rle)) throw new Exception("\r\nInput string has the wrong syntax! There isn't any char (e.g. 'a'->'z') in your input string to decode, there was/were only number(s).\r\n");

        return str_rle;
    }

    internal double GetPercentage(double x, double y)
    {
        return (100 * (x - y)) / x;
    }

    public void Dispose()
    {
        if (str_rle != null || str_base != null)
        {
            str_rle = str_base = null;
        }
    }
}

class Program
{
    private static string str_welcome = "\r\nRLE encoding/decoding tool, Oleg Orlov 2013(c)",
        str_notice = "Please, use the next syntax: <action> <string> (e.g. \"encode my_string\" or \"decode my_string\").\r\n\r\nWarning! The 2nd parameter (the string for encoding/decoding) must not content any whitespaces!";

    private static void EncodeString(ref string str)
    {
        using (RLE inst_rle = new RLE())
        {
            string str_encoded = inst_rle.Encode(ref str);
            Console.WriteLine("\r\nBase string ({0} chars): {1}\r\nAfter RLE-encoding ({2} chars): {3}\r\nCompression percentage: %{4}",
                                str.Length, str, str_encoded.Length, str_encoded,
                                inst_rle.GetPercentage((double)str.Length, (double)str_encoded.Length).ToString());
        }
    }

    private static void DecodeString(ref string str)
    {
        using (RLE inst_rle = new RLE())
        {
            string str_decoded = inst_rle.Decode(ref str);
            Console.WriteLine("\r\nBase string ({0} chars): {1}\r\nAfter RLE-decoding ({2} chars): {3}",
                    str.Length, str, str_decoded.Length, str_decoded);
        }
    }

    [STAThread]
    public static void Main(string[] args)
    {
        try
        {
            Console.WriteLine(str_welcome);

            if (args.Length != 0)
            {
                if (args[0] == "encode")
                {
                    EncodeString(ref args[1]);
                }
                else if (args[0] == "decode")
                {
                    DecodeString(ref args[1]);
                }
                else
                {
                    throw (new Exception("\r\nThere are only two methods: encode, decode. No other actions are available.\r\n"
                        + str_notice + "\r\n"));
                }
            }
            else
            {
                Console.WriteLine(str_notice);
                Environment.Exit(1);
            }
        }
        catch (Exception exc)
        {
            Console.WriteLine("\r\n{0}", exc);
            Environment.Exit(1);
        }
    }
}